use scraper::Html;

use super::{Extractor, trim_whitespace, parse_date};

#[derive(Debug, Default)]
pub struct Artist {
    pub artist_name: String,
    pub artist_url: String,
    pub date: String, 
    pub author: String,
    pub author_url: String,
}

impl Extractor for Artist {
    type Output = Self;
    fn extract_all(document: Html) -> Vec<Self> {
        let mut artists : Vec<Self> = Vec::new();

        let selector = scraper::Selector::parse("ul>li").unwrap();
        let select = document.select(&selector);
        for el in select {
            artists.push(Self::extract(Html::parse_fragment(&el.html())));
        }

        artists
    }

    fn extract(document: Html) -> Self {
      let a_el = document.select(&scraper::Selector::parse("a").unwrap()).next().unwrap();

      let title = a_el.inner_html();
      let url = a_el.value().attr("href").unwrap();

      let more_info_el = document.select(&scraper::Selector::parse("div").unwrap()).next().unwrap();
      let date = more_info_el.inner_html();

      let author_el = more_info_el.select(&scraper::Selector::parse("a").unwrap()).next().unwrap();
      let author = author_el.inner_html();
      let author_url = author_el.value().attr("href").unwrap();

      Self {
          artist_name: trim_whitespace(&title),
          artist_url: trim_whitespace(url),
          date: parse_date(&date),
          author: trim_whitespace(&author),
          author_url: trim_whitespace(author_url),
      }
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    const EXAMPLES: &'static str = 
r#"
<ul>
  <li>
    <a class="symbol__artist" href="/artists/babykingdom/">BabyKingdom</a>
    <div class="h5 any--no-wrap">
      2022-06-16 14:12:03 by 
      <a class="user a--inherit" data-icon="" data-is-vip="1" href="/users/suji/">suji</a>
    </div>
  </li>
  <li>
    <a class="symbol__artist" href="/artists/hiro-ruvish/">HIRO</a>
    <div class="h5 any--no-wrap">
      2022-06-16 11:11:06 by 
      <a class="user a--inherit" data-icon="" data-is-vip="1" href="/users/kumika/">kumika</a>
    </div>
  </li>
</ul>
"#;

  #[test]
  fn extract_all_must_return_two_result() {
    let document = scraper::Html::parse_fragment(EXAMPLES);
    let artists = Artist::extract_all(document);
    assert_eq!(artists.len(), 2);
    assert_eq!(artists[0].author, "suji");
    assert_eq!(artists[0].author_url, "/users/suji/");
    assert_eq!(artists[0].artist_name, "BabyKingdom");
    assert_eq!(artists[0].artist_url, "/artists/babykingdom/");
    assert_eq!(artists[0].date, "2022-06-16 14:12:03");
    assert_eq!(artists[1].author, "kumika");
    assert_eq!(artists[1].author_url, "/users/kumika/");
    assert_eq!(artists[1].artist_name, "HIRO");
    assert_eq!(artists[1].artist_url, "/artists/hiro-ruvish/");
    assert_eq!(artists[1].date, "2022-06-16 11:11:06");
  }

  const EXAMPLE: &'static str = r#"
  <a class="symbol__artist" href="/artists/babykingdom/">BabyKingdom</a>
    <div class="h5 any--no-wrap">
      2022-06-16 14:12:03 by 
      <a class="user a--inherit" data-icon="" data-is-vip="1" href="/users/suji/">suji</a>
    </div>
  "#;

  #[test]
  fn extract_must_be_return_correct_value() {
    let document = scraper::Html::parse_fragment(EXAMPLE);
    let artist = Artist::extract(document);
    assert_eq!(artist.author, "suji");
    assert_eq!(artist.author_url, "/users/suji/");
    assert_eq!(artist.artist_name, "BabyKingdom");
    assert_eq!(artist.artist_url, "/artists/babykingdom/");
    assert_eq!(artist.date, "2022-06-16 14:12:03");
  }
}